namespace Interpreter.Tree
{

	public enum OpType {Plus,Minus,Mult,Div};
	public enum OpSign {Plus,Minus};

	public class TreeNode
	{
		public virtual double Evalf()
		{
			return 0;
		}
	}
	
	public class Expression:TreeNode
	{
	}
	
	public class Const:Expression
	{
		double Val;
		public Const(double Val)
		{
			this.Val=Val;
		}
		public override double Evalf()
		{
			return Val;
		}
	}
	
	public class BinExpression:Expression
	{
		public Expression Left;
		public Expression Right;
		public OpType OperationType;
		public BinExpression(Expression Left,
			Expression Right,OpType OperationType)
		{
			this.Left=Left;
			this.Right=Right;
			this.OperationType=OperationType;
		}
		public override double Evalf()
		{
			switch(OperationType)
			{
				case OpType.Plus:return Left.Evalf()+Right.Evalf();
				case OpType.Minus:return Left.Evalf()-Right.Evalf();
				case OpType.Mult:return Left.Evalf()*Right.Evalf();
				case OpType.Div:return Left.Evalf()/Right.Evalf();
			}
			return 0;
		}
	}
	
	public class UnExpression:Expression
	{
		public Expression Expr;
		public OpSign Sign;
		public UnExpression(Expression Expr,OpSign Sign)
		{
			this.Expr=Expr;
			this.Sign=Sign;
		}
		public override double Evalf()
		{
			switch(Sign)
			{
				case OpSign.Plus:return Expr.Evalf();
				case OpSign.Minus:return -Expr.Evalf();
			}
			return 0;
		}

	}
}